@@ -7,6 +7,8 @@
 # --------------------------------------------------------------------------------------------------
 
 from tsfpga.module import BaseModule
+from tsfpga.vivado.project import VivadoNetlistProject
+from examples.tsfpga_example_env import get_tsfpga_modules
 
 
 class Module(BaseModule):
@@ -34,6 +36,17 @@ class Module(BaseModule):
         self.add_vunit_config(tb, generics=dict(pipeline=True))
         self.add_vunit_config(tb, generics=dict(pipeline=False))
 
+        # The setting of max_burst_length_beats is not really dependent on the clock configurations,
+        # so we do not need to test every possible combination of these settings.
+        tb = vunit_proj.library(self.library_name).test_bench("tb_axi_cdc")
+        tb.add_config(
+            name="input_clk_fast", generics=dict(input_clk_fast=True, max_burst_length_beats=16)
+        )
+        tb.add_config(
+            name="output_clk_fast", generics=dict(output_clk_fast=True, max_burst_length_beats=256)
+        )
+        tb.add_config(name="same_clocks", generics=dict(max_burst_length_beats=16))
+
         tb = vunit_proj.library(self.library_name).test_bench("tb_axil_cdc")
         tb.add_config(name="master_clk_fast", generics=dict(master_clk_fast=True))
         tb.add_config(name="slave_clk_fast", generics=dict(slave_clk_fast=True))
@@ -49,5 +62,41 @@ class Module(BaseModule):
         tb.add_config(name="axi", generics=dict(test_axi_lite=False))
 
         tb = vunit_proj.library(self.library_name).test_bench("tb_axil_mux")
-        tb.test("read_from_non_existent_slave_base_adress").set_generic("use_axil_bfm", False)
-        tb.test("write_to_non_existent_slave_base_adress").set_generic("use_axil_bfm", False)
+        tb.test("read_from_non_existent_slave_base_address").set_generic("use_axil_bfm", False)
+        tb.test("write_to_non_existent_slave_base_address").set_generic("use_axil_bfm", False)
+
+    def get_build_projects(self):
+        projects = []
+        modules = get_tsfpga_modules(names_include=[self.name, "common", "math"])
+        part = "xc7z020clg400-1"
+
+        generics = dict(
+            data_fifo_depth=1024,
+            max_burst_length_beats=256,
+            id_width=6,
+            addr_width=32,
+            full_ar_throughput=False,
+            full_aw_throughput=False,
+        )
+
+        projects.append(
+            VivadoNetlistProject(
+                name=f"{self.library_name}.axi_read_throttle",
+                modules=modules,
+                part=part,
+                top="axi_read_throttle",
+                generics=generics,
+            )
+        )
+
+        projects.append(
+            VivadoNetlistProject(
+                name=f"{self.library_name}.axi_write_throttle",
+                modules=modules,
+                part=part,
+                top="axi_write_throttle",
+                generics=generics,
+            )
+        )
+
+        return projects
