@@ -38,6 +38,9 @@ class VivadoProject:
         defined_at=None,
     ):
         """
+        Class constructor. Performs a shallow copy of the mutable arguments, so that the user
+        can e.g. append items to their list after creating an object.
+
         Arguments:
             name (str): Project name.
             modules (list(:class:`Module <.BaseModule>`)): Modules that shall be included in the
@@ -65,60 +68,24 @@ class VivadoProject:
                 projects set up.
         """
         self.name = name
-        self.modules = modules
+        self.modules = modules.copy()
         self.part = part
         self.static_generics = None if generics is None else generics.copy()
-        self.constraints = constraints
+        self.constraints = [] if constraints is None else constraints.copy()
+        self.tcl_sources = [] if tcl_sources is None else tcl_sources.copy()
+        self.build_step_hooks = [] if build_step_hooks is None else build_step_hooks.copy()
         self.default_run_index = default_run_index
         self.defined_at = defined_at
 
         # Will be set by child class when applicable
         self.is_netlist_build = False
-        self.analyze_clock_interaction = True
+        self.analyze_synthesis_timing = True
 
         self.top = name + "_top" if top is None else top
         self._vivado_path = vivado_path
 
-        self._setup_tcl_sources_list(tcl_sources)
-        self._setup_build_step_hooks(build_step_hooks)
-
         self.tcl = VivadoTcl(name=self.name)
 
-    def _setup_tcl_sources_list(self, tcl_sources_from_user):
-        tsfpga_tcl_sources = [
-            TSFPGA_TCL / "vivado_default_run.tcl",
-            TSFPGA_TCL / "vivado_fast_run.tcl",
-            TSFPGA_TCL / "vivado_messages.tcl",
-        ]
-        # Note that the list concatenation below does a (shallow) copy of the lists
-        user_tcl_sources = [] if tcl_sources_from_user is None else tcl_sources_from_user
-
-        # Add tsfpga TCL sources first. The user might want to change something in the tsfpga
-        # settings. Conversely, tsfpga should not modify something that the user has set up.
-        self.tcl_sources = tsfpga_tcl_sources + user_tcl_sources
-
-    def _setup_build_step_hooks(self, build_step_hooks_from_user):
-        # Lists are mutable. Since we assign and modify this one we have to copy it.
-        self.build_step_hooks = (
-            [] if build_step_hooks_from_user is None else build_step_hooks_from_user.copy()
-        )
-
-        self.build_step_hooks.append(
-            BuildStepTclHook(
-                TSFPGA_TCL / "vivado_report_utilization.tcl", "STEPS.SYNTH_DESIGN.TCL.POST"
-            )
-        )
-        self.build_step_hooks.append(
-            BuildStepTclHook(
-                TSFPGA_TCL / "vivado_report_utilization.tcl", "STEPS.WRITE_BITSTREAM.TCL.PRE"
-            )
-        )
-        self.build_step_hooks.append(
-            BuildStepTclHook(
-                TSFPGA_TCL / "vivado_check_timing.tcl", "STEPS.WRITE_BITSTREAM.TCL.PRE"
-            )
-        )
-
     def project_file(self, project_path):
         """
         Arguments:
@@ -128,6 +95,41 @@ class VivadoProject:
         """
         return project_path / (self.name + ".xpr")
 
+    def _setup_tcl_sources(self):
+        tsfpga_tcl_sources = [
+            TSFPGA_TCL / "vivado_default_run.tcl",
+            TSFPGA_TCL / "vivado_fast_run.tcl",
+            TSFPGA_TCL / "vivado_messages.tcl",
+        ]
+
+        # Add tsfpga TCL sources first. The user might want to change something in the tsfpga
+        # settings. Conversely, tsfpga should not modify something that the user has set up.
+        self.tcl_sources = tsfpga_tcl_sources + self.tcl_sources
+
+    def _setup_build_step_hooks(self):
+        # Check the implemented timing and resource utilization via TCL build hooks.
+        # This is different than for synthesis, where it is embedded in the build script.
+        # This is due to Vivado limitations related to post-synthesis hooks.
+        # Spceifically, the report_utilization figures do not include IP cores when it is run in
+        # a post-synthesis hook.
+        self.build_step_hooks.append(
+            BuildStepTclHook(TSFPGA_TCL / "report_utilization.tcl", "STEPS.WRITE_BITSTREAM.TCL.PRE")
+        )
+        self.build_step_hooks.append(
+            BuildStepTclHook(TSFPGA_TCL / "check_timing.tcl", "STEPS.WRITE_BITSTREAM.TCL.PRE")
+        )
+
+        if not self.analyze_synthesis_timing:
+            # In this special case however, the synthesized design is never opened. So in order to
+            # get a utiliztion report anyway we add it as a hook. This mode is exclusively used by
+            # netlist builds, which very rarely include IP cores, so it is acceptable that the
+            # utilization report might be erroneous with regards to IP cores.
+            self.build_step_hooks.append(
+                BuildStepTclHook(
+                    TSFPGA_TCL / "report_utilization.tcl", "STEPS.SYNTH_DESIGN.TCL.POST"
+                )
+            )
+
     def _create_tcl(self, project_path, ip_cache_path):
         """
         Make a TCL file that creates a Vivado project
@@ -167,6 +169,9 @@ class VivadoProject:
             bool: True if everything went well.
         """
         print(f"Creating Vivado project in {project_path}")
+        self._setup_tcl_sources()
+        self._setup_build_step_hooks()
+
         if not self.pre_create(project_path=project_path, ip_cache_path=ip_cache_path):
             print("ERROR: Project pre-create hook returned False. Failing the build.")
             return False
@@ -222,7 +227,7 @@ class VivadoProject:
             run_index=run_index,
             generics=all_generics,
             synth_only=synth_only,
-            analyze_clock_interaction=self.analyze_clock_interaction,
+            analyze_synthesis_timing=self.analyze_synthesis_timing,
         )
         create_file(build_vivado_project_tcl, tcl)
 
@@ -391,18 +396,18 @@ class VivadoProject:
         return UtilizationParser.get_size(report_as_string)
 
     def __str__(self):
-        result = str(self.__class__.__name__)
+        result = self.name
         if self.defined_at is not None:
-            result += f" defined at: {self.defined_at.resolve()}"
-        result += "\nName:      " + self.name
-        result += "\nTop level: " + self.top
+            result += f"\nDefined at: {self.defined_at.resolve()}"
+        result += f"\nType:       {self.__class__.__name__}"
+        result += f"\nTop level:  {self.top}"
         if self.static_generics is None:
             generics = "-"
         else:
             generics = ", ".join(
                 [f"{name}={value}" for name, value in self.static_generics.items()]
             )
-        result += "\nGenerics:  " + generics
+        result += f"\nGenerics:   {generics}"
 
         return result
 
@@ -412,14 +417,15 @@ class VivadoNetlistProject(VivadoProject):
     Used for handling Vivado build of a module without top level pinning.
     """
 
-    def __init__(self, analyze_clock_interaction=False, result_size_checkers=None, **kwargs):
+    def __init__(self, analyze_synthesis_timing=False, result_size_checkers=None, **kwargs):
         """
         Arguments:
-            analyze_clock_interaction (bool): Analysis of clock interaction, i.e. checking
-                for unhandled clock crossings, is disabled by default. Enabling it will add
-                significant build time (can be as much as +40%). Also, in order for clock
-                interction check to work, the clocks have to be created using a constraint
-                file.
+            analyze_synthesis_timing (bool): Enable analysis of the synthesized design's timing.
+                This will make the build flow open the design, and check for unhandled clock
+                crossings and pulse width violations.
+                Enabling it will add significant build time (can be as much as +40%).
+                Also, in order for clock crossing check to work, the clocks have to be created
+                using a constraint file.
             result_size_checkers (list(:class:`.SizeChecker`)): Size checkers that will be
                 executed after a succesful build. Is used to automatically check that
                 resource utilization is not greater than expected.
@@ -428,7 +434,7 @@ class VivadoNetlistProject(VivadoProject):
         super().__init__(**kwargs)
 
         self.is_netlist_build = True
-        self.analyze_clock_interaction = analyze_clock_interaction
+        self.analyze_synthesis_timing = analyze_synthesis_timing
         self.result_size_checkers = [] if result_size_checkers is None else result_size_checkers
 
     def build(self, **kwargs):  # pylint: disable=arguments-differ
