@@ -242,20 +242,22 @@ class VivadoTcl:
         run_index,
         generics=None,
         synth_only=False,
-        analyze_clock_interaction=True,
+        analyze_synthesis_timing=True,
     ):
-        # Max value in Vivado 2017.4. set_param will give an error if higher number.
-        num_threads = min(num_threads, 8)
+        # Max value in Vivado 2018.3+. set_param will give an error if higher number.
+        num_threads_general = min(num_threads, 32)
+        num_threads_synth = min(num_threads, 8)
 
         synth_run = f"synth_{run_index}"
         impl_run = f"impl_{run_index}"
 
         tcl = f"open_project {to_tcl_path(project_file)}\n"
-        tcl += f"set_param general.maxThreads {num_threads}\n"
+        tcl += f"set_param general.maxThreads {num_threads_general}\n"
+        tcl += f"set_param synth.maxThreads {num_threads_synth}\n"
         tcl += "\n"
         tcl += self._add_generics(generics)
         tcl += "\n"
-        tcl += self._synthesis(synth_run, num_threads, analyze_clock_interaction)
+        tcl += self._synthesis(synth_run, num_threads, analyze_synthesis_timing)
         tcl += "\n"
         if not synth_only:
             tcl += self._run(impl_run, num_threads, to_step="write_bitstream")
@@ -265,45 +267,63 @@ class VivadoTcl:
         tcl += "exit\n"
         return tcl
 
-    def _synthesis(self, run, num_threads, analyze_clock_interaction):
+    def _synthesis(self, run, num_threads, analyze_synthesis_timing):
         tcl = self._run(run, num_threads)
-        if analyze_clock_interaction:
-            tcl += "\n"
-            tcl += f"open_run {run}\n"
-            tcl += f"set run_directory [get_property DIRECTORY [get_runs {run}]]\n"
-            tcl += "\n"
-            tcl += (
-                r"if {[regexp {\(unsafe\)} "
-                "[report_clock_interaction -delay_type min_max -return_string]]} "
-            )
-            tcl += "{\n"
-            tcl += (
-                f'  puts "ERROR: Unhandled clock crossing in {run} run. '
-                'See reports in ${run_directory}"\n'
-            )
-            tcl += "\n"
-            tcl += '  set output_file [file join ${run_directory} "clock_interaction.rpt"]\n'
-            tcl += "  report_clock_interaction -delay_type min_max -file ${output_file}\n"
-            tcl += "\n"
-            tcl += '  set output_file [file join ${run_directory} "timing_summary.rpt"]\n'
-            tcl += "  report_timing_summary -file ${output_file}\n"
-            tcl += "\n"
-            tcl += "  exit 1\n"
-            tcl += "}\n"
+        if analyze_synthesis_timing:
+            # For synthesis flow we perform the timing checks by opening the design. It would have
+            # been more efficient to use a post-synthesis hook (since the design would already be
+            # open), if that mechanism had worked. It seems to be very bugged. So we add the
+            # checkers to the build script.
+            # For implementation, we use a pre-bitstream build hook which seems to work decently.
+            tcl += """
+open_run ${run}
+set run_directory [get_property DIRECTORY [get_runs ${run}]]
+
+# This call is duplicated in report_utilization.tcl for implementation.
+set output_file [file join ${run_directory} "hierarchical_utilization.rpt"]
+report_utilization -hierarchical -hierarchical_depth 4 -file ${output_file}
+
+
+# After synthesis we check for unhandled clock crossings and abort the build based on the result.
+# Other timing checks, e.g. setup/hold/pulse width violations, are not reliable after synthesis,
+# and should not abort the build. These need to be checked after implementation.
+"""
+
+            tcl += """
+# This code is duplicated in check_timing.tcl for implementation.
+if {[regexp {\\(unsafe\\)} [report_clock_interaction -delay_type min_max -return_string]]} {
+  puts "ERROR: Unhandled clock crossing in ${run} run. See clock_interaction.rpt and \
+timing_summary.rpt in ${run_directory}."
+
+  set output_file [file join ${run_directory} "clock_interaction.rpt"]
+  report_clock_interaction -delay_type min_max -file ${output_file}
+
+  set output_file [file join ${run_directory} "timing_summary.rpt"]
+  report_timing_summary -file ${output_file}
+
+  exit 1
+}
+"""
         return tcl
 
     @staticmethod
     def _run(run, num_threads, to_step=None):
         to_step = "" if to_step is None else " -to_step " + to_step
 
-        tcl = f"reset_run {run}\n"
-        tcl += f"launch_runs {run} -jobs {num_threads}{to_step}\n"
-        tcl += f"wait_on_run {run}\n"
-        tcl += "\n"
-        tcl += 'if {[get_property PROGRESS [get_runs %s]] != "100%%"} {\n' % run
-        tcl += f'  puts "ERROR: Run {run} failed."\n'
-        tcl += "  exit 1\n"
-        tcl += "}\n"
+        tcl = f"""
+set run {run}
+reset_run ${{run}}
+launch_runs ${{run}} -jobs {num_threads}{to_step}
+"""
+
+        tcl += """
+wait_on_run ${run}
+
+if {[get_property PROGRESS [get_runs ${run}]] != "100%"} {
+  puts "ERROR: Run ${run} failed."
+  exit 1
+}
+"""
         return tcl
 
     def _write_hw_platform(self, output_path):
